#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Automaton/Automaton.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
#include <CompuCell3D/Potts3D/CellInventory.h>
#include <CompuCell3D/Field3D/WatchableField3D.h>
#include <CompuCell3D/Field3D/Field3DImpl.h>
#include <CompuCell3D/Field3D/Field3D.h>
#include <CompuCell3D/Field3D/Field3DIO.h>
#include <BasicUtils/BasicClassGroup.h>



#include <BasicUtils/BasicString.h>
#include <BasicUtils/BasicException.h>
#include <BasicUtils/BasicRandomNumberGenerator.h>
#include <PublicUtilities/StringUtils.h>
#include <string>
#include <cmath>
#include <iostream>
#include <fstream>
#include <sstream>


#include <time.h>


#include "SteadyStateDiffusionSolver.h"
#include "hpppdesolvers.h"//have to put this header last to avoid STL header clash on linux

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// std::ostream & operator<<(std::ostream & out,CompuCell3D::DiffusionData & diffData){
//    
//    
// }


using namespace CompuCell3D;
using namespace std;


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SteadyStateDiffusionSolverSerializer::serialize(){

	for(int i = 0 ; i < solverPtr->diffSecrFieldTuppleVec.size() ; ++i){
		ostringstream outName;

		outName<<solverPtr->diffSecrFieldTuppleVec[i].diffData.fieldName<<"_"<<currentStep<<"."<<serializedFileExtension;
		ofstream outStream(outName.str().c_str());
		solverPtr->outputField( outStream,solverPtr->concentrationFieldVector[i]);
	}

}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SteadyStateDiffusionSolverSerializer::readFromFile(){
	try{
		for(int i = 0 ; i < solverPtr->diffSecrFieldTuppleVec.size() ; ++i){
			ostringstream inName;
			inName<<solverPtr->diffSecrFieldTuppleVec[i].diffData.fieldName<<"."<<serializedFileExtension;

			solverPtr->readConcentrationField(inName.str().c_str(),solverPtr->concentrationFieldVector[i]);;
		}

	} catch (BasicException &e) {
		cerr<<"COULD NOT FIND ONE OF THE FILES"<<endl;
		throw BasicException("Error in reading diffusion fields from file ",e);
	}


}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
SteadyStateDiffusionSolver::SteadyStateDiffusionSolver()
: DiffusableVectorFortran<Array3DLinearFortranField3DAdapter<float> >(),deltaX(1.0),deltaT(1.0)
{
	serializerPtr=0;
	serializeFlag=false;
	readFromFileFlag=false;
	//haveCouplingTerms=false;
	serializeFrequency=0;
	//boxWatcherSteppable=0;
	//    useBoxWatcher=false;

}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
SteadyStateDiffusionSolver::~SteadyStateDiffusionSolver()
{

	if(serializerPtr)
		delete serializerPtr ; serializerPtr=0;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SteadyStateDiffusionSolver::init(Simulator *simulator, CC3DXMLElement *_xmlData) {




	simPtr=simulator;
	potts = simulator->getPotts();
	automaton=potts->getAutomaton();

	///getting cell inventory
	cellInventoryPtr=& potts->getCellInventory(); 

	///getting field ptr from Potts3D
	cellFieldG=(WatchableField3D<CellG *> *)potts->getCellFieldG();
	fieldDim=cellFieldG->getDim();

	update(_xmlData,true);



	///setting member function pointers
	diffusePtr=&SteadyStateDiffusionSolver::diffuse;
	secretePtr=&SteadyStateDiffusionSolver::secrete;


	numberOfFields=diffSecrFieldTuppleVec.size();


	vector<string> concentrationFieldNameVectorTmp; //temporary vector for field names
	///assign vector of field names
	concentrationFieldNameVectorTmp.assign(diffSecrFieldTuppleVec.size(),string(""));

	cerr<<"diffSecrFieldTuppleVec.size()="<<diffSecrFieldTuppleVec.size()<<endl;

	for(unsigned int i = 0 ; i < diffSecrFieldTuppleVec.size() ; ++i){
		//       cerr<<" concentrationFieldNameVector[i]="<<diffDataVec[i].fieldName<<endl;
		//       concentrationFieldNameVector.push_back(diffDataVec[i].fieldName);
		concentrationFieldNameVectorTmp[i] = diffSecrFieldTuppleVec[i].diffData.fieldName;
		cerr<<" concentrationFieldNameVector[i]="<<concentrationFieldNameVectorTmp[i]<<endl;
	}

	// //setting up couplingData - field-field interaction terms
	// vector<CouplingData>::iterator pos;

	// for(unsigned int i = 0 ; i < diffSecrFieldTuppleVec.size() ; ++i){
		// pos=diffSecrFieldTuppleVec[i].diffData.couplingDataVec.begin();
		// for(int j = 0 ; j < diffSecrFieldTuppleVec[i].diffData.couplingDataVec.size() ; ++j){

			// for(int idx=0; idx<concentrationFieldNameVectorTmp.size() ; ++idx){
				// if( concentrationFieldNameVectorTmp[idx] == diffSecrFieldTuppleVec[i].diffData.couplingDataVec[j].intrFieldName ){
					// diffSecrFieldTuppleVec[i].diffData.couplingDataVec[j].fieldIdx=idx;
					// haveCouplingTerms=true; //if this is called at list once we have already coupling terms and need to proceed differently with scratch field initialization
					// break;
				// }
				// //this means that required interacting field name has not been found
				// if( idx == concentrationFieldNameVectorTmp.size()-1 ){
					// //remove this interacting term
					// //                pos=&(diffDataVec[i].degradationDataVec[j]);
					// diffSecrFieldTuppleVec[i].diffData.couplingDataVec.erase(pos);
				// }
			// }
			// ++pos;
		// }
	// } 


	cerr<<"FIELDS THAT I HAVE"<<endl; 
	for(unsigned int i = 0 ; i < diffSecrFieldTuppleVec.size() ; ++i){
		cerr<<"Field "<<i<<" name: "<<concentrationFieldNameVectorTmp[i]<<endl;
	}




	cerr<<"FlexibleDiffusionSolverFE: extra Init in read XML"<<endl;

	workFieldDim=Dim3D(fieldDim.x+1,fieldDim.y+1,fieldDim.z+1);
	///allocate fields including scrartch field
    allocateDiffusableFieldVector(diffSecrFieldTuppleVec.size(),fieldDim); 
    //scratch vecor
	int maxLMN=fieldDim.x+1;
	if(maxLMN<fieldDim.y+1)
		maxLMN=fieldDim.y+1;
	if(maxLMN<fieldDim.z+1)
		maxLMN=fieldDim.z+1;


    scratchVec.assign(30 + fieldDim.x+1+1 + fieldDim.y+1+1 + 5*(fieldDim.z+1)+maxLMN+7*((int)(fieldDim.x+1+1)+(int)(fieldDim.y+1+1)),0.0);
    scratch=&(scratchVec[0]);
    
	cerr<<"fieldDim="<<fieldDim<<endl;	
	//vectors used to specify boundary conditions
	bdaVec.assign((fieldDim.y+1)*(fieldDim.z+1),0.0);
	bdbVec.assign((fieldDim.y+1)*(fieldDim.z+1),0.0);
	bdcVec.assign((fieldDim.x+1)*(fieldDim.z+1),0.0);
	bddVec.assign((fieldDim.x+1)*(fieldDim.z+1),0.0);

	bdeVec.assign((fieldDim.x+1)*(fieldDim.y+1),0.0);
	bdfVec.assign((fieldDim.x+1)*(fieldDim.y+1),0.0);


	// if(!haveCouplingTerms){
		// allocateDiffusableFieldVector(diffSecrFieldTuppleVec.size()+1,workFieldDim); //+1 is for additional scratch field
	// }else{
		// allocateDiffusableFieldVector(2*diffSecrFieldTuppleVec.size(),workFieldDim); //with coupling terms every field need to have its own scratch field
	// }

	//here I need to copy field names from concentrationFieldNameVectorTmp to concentrationFieldNameVector
	//because concentrationFieldNameVector is reallocated with default values once I call allocateDiffusableFieldVector


	for(unsigned int i=0 ; i < concentrationFieldNameVectorTmp.size() ; ++i){
		concentrationFieldNameVector[i]=concentrationFieldNameVectorTmp[i];
	}


	//register fields once they have been allocated
	for(unsigned int i = 0 ; i < diffSecrFieldTuppleVec.size() ; ++i){
		simPtr->registerConcentrationField(concentrationFieldNameVector[i] , concentrationFieldVector[i]);
		cerr<<"registring field: "<<concentrationFieldNameVector[i]<<" field address="<<concentrationFieldVector[i]<<endl;
	}





	//    exit(0);

	periodicBoundaryCheckVector.assign(3,false);
	string boundaryName;
	boundaryName=potts->getBoundaryXName();
	changeToLower(boundaryName);
	if(boundaryName=="periodic")  {
		periodicBoundaryCheckVector[0]=true; 
	}
	boundaryName=potts->getBoundaryYName();
	changeToLower(boundaryName);
	if(boundaryName=="periodic")  {
		periodicBoundaryCheckVector[1]=true;
	}

	boundaryName=potts->getBoundaryZName();
	changeToLower(boundaryName);
	if(boundaryName=="periodic")  {
		periodicBoundaryCheckVector[2]=true; 
	}


	simulator->registerSteerableObject(this);

}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SteadyStateDiffusionSolver::extraInit(Simulator *simulator){

	if((serializeFlag || readFromFileFlag) && !serializerPtr){
		serializerPtr=new SteadyStateDiffusionSolverSerializer();
		serializerPtr->solverPtr=this;
	}

	if(serializeFlag){
		simulator->registerSerializer(serializerPtr);
	}

	//checking if box watcher is necessary at all


	 bool pluginAlreadyRegisteredFlag;

	 Plugin * centerOfMassPlugin=Simulator::pluginManager.get("CenterOfMass",&pluginAlreadyRegisteredFlag);
	 if(!pluginAlreadyRegisteredFlag)
		 centerOfMassPlugin->init(simulator);

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SteadyStateDiffusionSolver::start() {
	//     if(diffConst> (1.0/6.0-0.05) ){ //hard coded condtion for stability of the solutions - assume dt=1 dx=dy=dz=1
	// 
	//       cerr<<"CANNOT SOLVE DIFFUSION EQUATION: STABILITY PROBLEM - DIFFUSION CONSTANT TOO LARGE. EXITING..."<<endl;
	//       exit(0);
	// 
	//    }


	dt_dx2=deltaT/(deltaX*deltaX);
	if(readFromFileFlag){
		try{

			//          serializerPtr->readFromFile();

		} catch (BasicException &e){
			cerr<<"Going to fail-safe initialization"<<endl;
			initializeConcentration(); //if there was error, initialize using failsafe defaults
		}

	}else{
		initializeConcentration();//Normal reading from User specified files
	}

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SteadyStateDiffusionSolver::initializeConcentration(){


	for(unsigned int i = 0 ; i <diffSecrFieldTuppleVec.size() ; ++i){
		if(diffSecrFieldTuppleVec[i].diffData.concentrationFileName.empty()) continue;
		cerr<<"fail-safe initialization "<<diffSecrFieldTuppleVec[i].diffData.concentrationFileName<<endl;
		readConcentrationField(diffSecrFieldTuppleVec[i].diffData.concentrationFileName,concentrationFieldVector[i]);
	}


}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SteadyStateDiffusionSolver::step(const unsigned int _currentStep) {

	currentStep=_currentStep;

	(this->*secretePtr)();

	(this->*diffusePtr)();


	if(serializeFrequency>0 && serializeFlag && !(_currentStep % serializeFrequency)){
		serializerPtr->setCurrentStep(currentStep);
		serializerPtr->serialize();
	}

}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SteadyStateDiffusionSolver::secreteOxygenSingleField(unsigned int idx){
	
	ConcentrationField_t * concentrationFieldPtr=concentrationFieldVector[idx];
	vector<float> & containerRef=concentrationFieldPtr->getContainerRef();

	 //containerRef.assign(containerRef.size(),0.0); //zero concentration vector	


	CellInventory  * cellInventoryPtr=& (potts->getCellInventory());

	CellInventory::cellInventoryIterator cInvItr;
	CellG * cell;



	Point3D pt;

	int flag=0;
	int flag2=0;
	float sumconcen=0.0;
	float numpoints=0.0;
	float aveconc=0.0;
	float sumconcen2=0.0;
	float numpoints2=0.0;
	float aveconc2=0.0;
	float tmpsec=0.0;
	float tmpsec2=0.0;
	float maxconc=0.0;
	float maxconc2=0.0;

	
	//SecretionData & secrData=diffSecrFieldTuppleVec[idx].secrData;

	vector<OxygenSecretionParameters> & oxygenSecretionData=diffSecrFieldTuppleVec[idx].oxygenSecretionData;
        DiffusionData & diffData = diffSecrFieldTuppleVec[idx].diffData;
	float diffConst=diffData.diffConst;
	//std::map<unsigned char,float>::iterator mitr;	
	//std::map<unsigned char,float>::iterator end_mitr=secrData.typeIdSecrConstMap.end();	
	//cerr<<"INSIDE OXYGEN SECRETION\n\n\n\n"<<endl;
	float currentConcentration;
	float newConcentration;
		for (int x = 0; x < fieldDim.x; x++)
		for (int y = 0; y < fieldDim.y; y++)
			for (int z = 0; z < fieldDim.z; z++){
				pt.x=x;
				pt.y=y;
				pt.z=z;
				cell=cellFieldG->get(pt);
				if(cell){ //do not secrete in medium				
					//mitr=secrData.typeIdSecrConstMap.find(cell->type);
					if(cell->type<oxygenSecretionData.size() && oxygenSecretionData[cell->type].dataInitialized){
						OxygenSecretionParameters & params=oxygenSecretionData[cell->type];
						currentConcentration=concentrationFieldPtr->get(pt);
						if (params.n ==1){
						  sumconcen+=currentConcentration;
						  numpoints+=1;
						  if ( currentConcentration > maxconc ){
						    maxconc = currentConcentration;
						  }
						  
						}
						if (params.n ==2 && z>5){
						  sumconcen2+=currentConcentration;
						  numpoints2+=1;
						  if ( currentConcentration > maxconc2 ){
						    maxconc2 = currentConcentration;
						  }
						}
						
							
						
					}

				}
			}
	// calculation of average PO2 at choriocapillaris and CNV		
	aveconc = sumconcen / numpoints;
	aveconc2 = sumconcen2 / numpoints2;
	cerr<<"aveconc="<<aveconc<<"numpoints="<<numpoints<<"maxConc="<<maxconc<<endl;
	cerr<<"aveconc2="<<aveconc2<<"numpoints2="<<numpoints2<<"maxConc="<<maxconc2<<endl;
	sumconcen=0;
	numpoints=0;
	sumconcen2=0;
	numpoints2=0;
	
	
			
	for (int x = 0; x < fieldDim.x; x++)
		for (int y = 0; y < fieldDim.y; y++)
			for (int z = 0; z < fieldDim.z; z++){
				pt.x=x;
				pt.y=y;
				pt.z=z;
				cell=cellFieldG->get(pt);
				if(cell){ //do not secrete in medium				
					//mitr=secrData.typeIdSecrConstMap.find(cell->type);
					if(cell->type<oxygenSecretionData.size() && oxygenSecretionData[cell->type].dataInitialized){
						OxygenSecretionParameters & params=oxygenSecretionData[cell->type];
						//choriocapillaris and CNV oxygen secretion 
						if (params.n ==1){
						    if (aveconc<params.pblood && flag==0){
						    params.bf=params.beta + params.bf;
						    tmpsec=params.bf;
						    flag=1;
						    }
						    if (aveconc>params.pblood && flag==0 && params.bf>0){
						    params.bf=-params.beta +params.bf;
						    tmpsec=params.bf;
						    flag=1;
						    }
						    concentrationFieldPtr->set(pt,-tmpsec);
						}
						if (params.n ==2){
						    if (aveconc2<params.pblood && flag2==0 && maxconc2 < maxconc){
						    params.bf=params.beta + params.bf;
						    tmpsec2=params.bf;
						    flag2=1;
						    }
						    if (aveconc2>params.pblood && flag2==0 && params.bf>0){
						    params.bf=-params.beta +params.bf;
						    tmpsec2=params.bf;
						    flag2=1;
						    }
						    concentrationFieldPtr->set(pt,-tmpsec2);
						}
						
						
						
								//minus sign is added because of the convention implied by pde solving fcn - deltaT scaling is done in diffuseSingleField function		 
						
						//concentrationFieldPtr->set(pt,-1.0);	
					}else{
						//concentrationFieldPtr->set(pt,0.0);
					}

				}else{
					concentrationFieldPtr->set(pt,0.0);
					
				}
			}
			 cerr<<" current_sec1  "<<tmpsec<<"    current_sec2  "<<tmpsec2<<endl;




}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SteadyStateDiffusionSolver::secreteSingleField(unsigned int idx){
	
	ConcentrationField_t * concentrationFieldPtr=concentrationFieldVector[idx];
	vector<float> & containerRef=concentrationFieldPtr->getContainerRef();

// 	containerRef.assign(containerRef.size(),0.0); //zero concentration vector//NOT compatible with OxygenSecretion


	CellInventory  * cellInventoryPtr=& (potts->getCellInventory());

	CellInventory::cellInventoryIterator cInvItr;
	CellG * cell;



	Point3D pt;

	

	float secrConst;
	float currentConcentration;
	SecretionData & secrData=diffSecrFieldTuppleVec[idx].secrData;
	DiffusionData & diffData = diffSecrFieldTuppleVec[idx].diffData;
	vector<OxygenSecretionParameters> & oxygenSecretionData=diffSecrFieldTuppleVec[idx].oxygenSecretionData;
	float diffConst=diffData.diffConst;
	std::map<unsigned char,float>::iterator mitr;	
	std::map<unsigned char,float>::iterator end_mitr=secrData.typeIdSecrConstMap.end();
	
	
		for (int x = 0; x < fieldDim.x; x++)
		for (int y = 0; y < fieldDim.y; y++)
			for (int z = 0; z < fieldDim.z; z++){
				pt.x=x;
				pt.y=y;
				pt.z=z;
				cell=cellFieldG->get(pt);
				if(cell){ //do not secrete in medium				
					//mitr=secrData.typeIdSecrConstMap.find(cell->type);
					if(cell->type<oxygenSecretionData.size() && oxygenSecretionData[cell->type].dataInitialized){//prevents from overwriting on oxygen secretion data

					}else{
						concentrationFieldPtr->set(pt,0.0);
					}
					

				}else{
					concentrationFieldPtr->set(pt,0.0);
					
				}
			}

	for (int x = 0; x < fieldDim.x; x++)
		for (int y = 0; y < fieldDim.y; y++)
			for (int z = 0; z < fieldDim.z; z++){
				pt.x=x;
				pt.y=y;
				pt.z=z;
				cell=cellFieldG->get(pt);
// 				currentConcentration=concentrationFieldPtr->get(pt);
// 				concentrationFieldPtr->set(pt,0.0);
				if(cell){ //do not secrete in medium				
					mitr=secrData.typeIdSecrConstMap.find(cell->type);
					
					
					
					if(mitr!=end_mitr){
					  if (mitr->second < 0.0 ){
					    
					    secrConst=-mitr->second;//*(currentConcentration/(currentConcentration+2.0));	//minus sign is added because of the convention implied by pde solving fcn - deltaT scaling is done in 
					    
// 					    if (currentConcentration < 0.0){
// 					      secrConst=0;
// 					    }
					    //cerr<<"pt="<<pt<<" secrConst="<<secrConst<<"Const="<<currentConcentration<<endl;
					  }else{
						secrConst=-mitr->second;
					  }
						
					  concentrationFieldPtr->set(pt,secrConst);	
						
					}		
				}else{
				      concentrationFieldPtr->set(pt,0.0);
					
					
				}
			}
			
	// Setting mixed boundary conditions---- Zero flux at z=0 and constant PO2 concentration at z=34. 		
 		for (int x = 0; x < fieldDim.x; x++)
 		for (int y = 0; y < fieldDim.y; y++){
 			
				pt.x=x;
				pt.y=y;
				pt.z=0;
				concentrationFieldPtr->set(pt,0);
				pt.z=34;
				concentrationFieldPtr->set(pt,18);
				
			}


	//for(cInvItr=cellInventoryPtr->cellInventoryBegin() ; cInvItr !=cellInventoryPtr->cellInventoryEnd() ;++cInvItr )
	//{
	//	cell=cellInventoryPtr->getCell(cInvItr);
	//	pt.x=(int)cell->xCM/(float)cell->volume;
	//	pt.y=(int)cell->yCM/(float)cell->volume;
	//	pt.z=(int)cell->zCM/(float)cell->volume;

	//	 mitr=secrData.typeIdSecrConstMap.find(cell->type);
	//	 if(mitr!=end_mitr){
	//		 secrConst=-mitr->second;	//minus sign is added because of the convention implied by pde solving fcn - deltaT scaling is done in diffuseSingleField function
	//		 concentrationFieldPtr->set(pt,secrConst);			 
	//	 }		
	//}

	// SecretionData & secrData=diffSecrFieldTuppleVec[idx].secrData;


	// std::map<unsigned char,float>::iterator mitr;	
	// std::map<unsigned char,float>::iterator end_mitr=secrData.typeIdSecrConstMap.end();
	// std::map<unsigned char,UptakeData>::iterator mitrUptake;
	// std::map<unsigned char,UptakeData>::iterator end_mitrUptake=secrData.typeIdUptakeDataMap.end();



	// CellG *currentCellPtr;
	// Field3DImpl<float> * concentrationField=concentrationFieldVector[idx];
	// float currentConcentration;
	// float secrConst;
	// float secrConstMedium=0.0;
	// float maxUptakeInMedium=0.0;
	// float relativeUptakeRateInMedium=0.0;

	// ConcentrationField_t * concentrationFieldPtr=concentrationFieldVector[idx];
	// Array2D_t & concentrationArray = concentrationFieldPtr->getContainer();


	// bool doUptakeFlag=false;
	// bool uptakeInMediumFlag=false;
	// bool secreteInMedium=false;
	// //the assumption is that medium has type ID 0
	// mitr=secrData.typeIdSecrConstMap.find(automaton->getTypeId("Medium"));

	// if( mitr != end_mitr){
		// secreteInMedium=true;
		// secrConstMedium=mitr->second;
	// }

	// //uptake for medium setup
	// if(secrData.typeIdUptakeDataMap.size()){
		// doUptakeFlag=true;
	// }
	// //uptake for medium setup
	// if(doUptakeFlag){
		// mitrUptake=secrData.typeIdUptakeDataMap.find(automaton->getTypeId("Medium"));
		// if(mitrUptake != end_mitrUptake){
			// maxUptakeInMedium=mitrUptake->second.maxUptake;
			// relativeUptakeRateInMedium=mitrUptake->second.relativeUptakeRate;
			// uptakeInMediumFlag=true;

		// }
	// }

	// Point3D pt;
	// for (int x = 1; x < workFieldDim.x-1; x++)
		// for (int y = 1; y < workFieldDim.y-1; y++){
			// pt=Point3D(x-1,y-1,0);
			// currentCellPtr=cellFieldG->get(pt);
			// currentConcentration = concentrationArray[x][y];

			// if(secreteInMedium && ! currentCellPtr){
				// concentrationArray[x][y]=currentConcentration+secrConstMedium;
			// }

			// if(currentCellPtr){
                                 // if(secrData.secretionTypeIds.find(currentCellPtr->type) == secrData.secretionTypeIds.end()){ 
                                    // continue;
                                 // }

				// mitr=secrData.typeIdSecrConstMap.find(currentCellPtr->type);
				// if(mitr!=end_mitr){
					// secrConst=mitr->second;
					// concentrationArray[x][y]=currentConcentration+secrConst;
				// }
			// }
				// if(doUptakeFlag){
					// if(uptakeInMediumFlag && ! currentCellPtr){
						// if(currentConcentration>maxUptakeInMedium){
							// concentrationArray[x][y]-=maxUptakeInMedium;	
						// }else{
							// concentrationArray[x][y]-= currentConcentration*relativeUptakeRateInMedium;	
						// }
					// }
					// if(currentCellPtr){
						
						// mitrUptake=secrData.typeIdUptakeDataMap.find(currentCellPtr->type);
						// if(mitrUptake!=end_mitrUptake){
							// if(currentConcentration > mitrUptake->second.maxUptake){
								// concentrationArray[x][y]-=mitrUptake->second.maxUptake;
								// //cerr<<" uptake concentration="<< currentConcentration<<" relativeUptakeRate="<<mitrUptake->second.relativeUptakeRate<<" subtract="<<mitrUptake->second.maxUptake<<endl;
							// }else{
								// //cerr<<"concentration="<< currentConcentration<<" relativeUptakeRate="<<mitrUptake->second.relativeUptakeRate<<" subtract="<<currentConcentration*mitrUptake->second.relativeUptakeRate<<endl;
								// concentrationArray[x][y]-= currentConcentration*mitrUptake->second.relativeUptakeRate;	
							// }
						// }
					// }
				// }
		// }


}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SteadyStateDiffusionSolver::secrete() {

	for(unsigned int i = 0 ; i < diffSecrFieldTuppleVec.size() ; ++i ){

		//handle Oxygen Secretion
		//cerr<<"diffSecrFieldTuppleVec[i].oxygenSecretionData.size()="<<diffSecrFieldTuppleVec[i].oxygenSecretionData.size()<<endl;
		if(diffSecrFieldTuppleVec[i].oxygenSecretionData.size()){ //if there is any oxygen secretion data specified for this field use oxygen secretion mode
			secreteOxygenSingleField(i);
//			continue; //ignore other secretion modes
		}
		for(unsigned int j = 0 ; j <diffSecrFieldTuppleVec[i].secrData.secretionFcnPtrVec.size() ; ++j){
			(this->*diffSecrFieldTuppleVec[i].secrData.secretionFcnPtrVec[j])(i);
		}
		


		

		

	}



}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SteadyStateDiffusionSolver::boundaryConditionInit(ConcentrationField_t *concentrationField){
	// Array_t & concentrationArray=concentrationField->getContainer();


	// if(periodicBoundaryCheckVector[0]){//periodic BC along X
		// for(int y=0 ; y<workFieldDim.y ; ++y){
			// concentrationArray[0][y]=concentrationArray[workFieldDim.x-2][y];
			// concentrationArray[workFieldDim.x-1][y]=concentrationArray[1][y];
		// }


	// }else{//noFlux BC along X
		// for(int y=0 ; y<workFieldDim.y ; ++y){
			// concentrationArray[0][y]=concentrationArray[1][y];
			// concentrationArray[workFieldDim.x-1][y]=concentrationArray[workFieldDim.x-2][y];
		// }


	// }

	// if(periodicBoundaryCheckVector[1]){//periodic BC along Y
		// for(int x=0 ; x<workFieldDim.x ; ++x){
			// concentrationArray[x][0]=concentrationArray[x][workFieldDim.y-2];
			// concentrationArray[x][workFieldDim.y-1]=concentrationArray[x][1];
		// }

	// }else{//noFlux BC along Y
		// for(int x=0 ; x<workFieldDim.x ; ++x){
			// concentrationArray[x][0]=concentrationArray[x][1];
			// concentrationArray[x][workFieldDim.y-1]=concentrationArray[x][workFieldDim.y-2];
		// }

	// }

	// //    if(periodicBoundaryCheckVector[0] || periodicBoundaryCheckVector[1]){
	// //       for(int y=0 ; y<workFieldDim.y ; ++y){
	// //             concentrationArray[0][y]=concentrationArray[workFieldDim.x-2][y];
	// //             concentrationArray[workFieldDim.x-1][y]=concentrationArray[1][y];
	// //       }
	// // 
	// //       for(int x=0 ; x<workFieldDim.x ; ++x){
	// //             concentrationArray[x][0]=concentrationArray[x][workFieldDim.y-2];
	// //             concentrationArray[x][workFieldDim.y-1]=concentrationArray[x][1];
	// //       }
	// // 
	// //       
	// //    }else{
	// //       for(int y=0 ; y<workFieldDim.y ; ++y){
	// //             concentrationArray[0][y]=concentrationArray[1][y];
	// //             concentrationArray[workFieldDim.x-1][y]=concentrationArray[workFieldDim.x-2][y];
	// //       }
	// // 
	// //       for(int x=0 ; x<workFieldDim.x ; ++x){
	// //             concentrationArray[x][0]=concentrationArray[x][1];
	// //             concentrationArray[x][workFieldDim.y-1]=concentrationArray[x][workFieldDim.y-2];
	// //       }
	// // 
	// // 
	// //    }


}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SteadyStateDiffusionSolver::diffuseSingleField(unsigned int idx){

float x_min,x_max;
integer l ,  lbdcnd  ;
float *bda, *bdb;
float y_min,y_max;
integer m,mbdcnd;
float *bdc, *bdd; 

float z_min,z_max;
integer n,nbdcnd;
float *bde, *bdf; 


float elmbda, *f;
integer ldimf;
integer mdimf;

float pertrb;
integer ierror;
float *w;

//dimensions - here we need to use l,m,n euqal to corresopnding dimension -1
x_min=0;
x_max=fieldDim.x;
l=(int)x_max-1;

y_min=0;
y_max=fieldDim.y;
m=(int)y_max-1;


z_min=0;
z_max=fieldDim.z;
n=(int)z_max-1;


//boundary conditions vectors
bda=&(bdaVec[0]);
bdb=&(bdbVec[0]);

bdc=&(bdcVec[0]);
bdd=&(bddVec[0]);

bde=&(bdeVec[0]);
bdf=&(bdfVec[0]);


if (periodicBoundaryCheckVector[0]){
	
	lbdcnd=0; //periodic boundary conditions -x
}else{
	lbdcnd=1; //the solution is specified at X = XS and X = XF 
}

if (periodicBoundaryCheckVector[1]){
	mbdcnd=0; //periodic boundary conditions -y
}else{	
	mbdcnd=1; //the solution is specified at Y = YS and Y = YF
}


if (periodicBoundaryCheckVector[2]){
	nbdcnd=0; //periodic boundary conditions -z
}else{	
	nbdcnd=4; //the derivative of the solution is specified at Z = ZS and and the solution Z = ZF
}



DiffusionData & diffData = diffSecrFieldTuppleVec[idx].diffData;
float diffConst=diffData.diffConst;
float decayConst=diffData.decayConst;
float deltaT=diffData.deltaT;
float deltaX=diffData.deltaX;
float dt_dx2=deltaT/(deltaX*deltaX);

//THIS IS THE EQUATION THAT IS BEING SOLVED
/*         (d/dX)(dU/dX) + (d/dY)(dU/dY) + (d/dZ)(dU/dZ) */

/*                    + LAMBDA*U = F(X,Y,Z) . */



//now have to do rescaling to put equation into a form required by PDE solving function
elmbda=-decayConst*deltaT/(dt_dx2*diffConst); // although this could be simplified I leave it this for to show where all the scaling factors come from

ConcentrationField_t * concentrationFieldPtr=concentrationFieldVector[idx];
vector<float> & containerRef=concentrationFieldPtr->getContainerRef();

//cerr<<"dt_dx2*diffConst"<<dt_dx2*diffConst<<endl;
//cerr<<"scratch.size()="<<scratchVec.size()<<endl;

for (int i  = 0 ; i < containerRef.size() ; ++i){
	containerRef[i]/=dt_dx2*diffConst;
	//containerRef[i]=0.0;

}

ldimf=x_max+1; //this is required by solving function
mdimf=y_max+1; //this is required by solving function

f=concentrationFieldPtr->getContainerArrayPtr();

w=scratch;
//int ix;
////ofstream outInit("initHPP.txt");
//for (int k = 0 ; k < z_max ; ++k)
//	for (int j = 0 ; j < y_max ; ++j)
//		for (int i = 0 ; i < x_max ; ++i){
//		
//		//ix=(j+1)*(x_max+1)+i+1;
//		ix=i+1 + (j+1+ (k+1) * (y_max+1)) * (x_max+1);
//		//if (i>48 &&i<52 && j>48 &&j<52){
//		//	f_vec[idx]=-exp(-((i-50)*(i-50)+(j-50)*(j-50))/(2*2.5*2.5)); 
//		//}
//		//if(j==0){
//		//	cerr<<"i="<<i<<" j="<<j<<" ix="<<ix<<endl;
//		//}
//		f[ix]=0.0;
//		if (i>8 &&i<12 && j>8 &&j<12 && k>8 &&k<12){
//			f[ix]=-exp(-((i-10)*(i-10)+(j-10)*(j-10)+(k-10)*(k-10))/(2*2.5*2.5)); 
//			//cerr<<i<<" " <<j<<" 0 "<<f[ix]<<endl;
//		}
//
//		//outInit<<i<<" " <<j<<" 0 "<<f[ix]<<endl;
//		
//		//if (i>10 &&i<20 && j>10 &&j<20){
//		//	f_vec[idx]=-exp(-((i-15)*(i-15)+(j-15)*(j-15))/(2*2.5*2.5)); //notice if you want secretion you need to put f term with minus sign
//		//															     // see fortran file for definitions - notice f fcn is on the right hand side
//		//	//f_vec[idx]=0.0;
//		//}
//
//		
//	}

	//CellG *cell;
	//Point3D pt;
	//float currentConcentration;
	//for (int x = 0; x < fieldDim.x; x++)
	//	for (int y = 0; y < fieldDim.y; y++)
	//		for (int z = 0; z < fieldDim.z; z++){
	//			pt.x=x;
	//			pt.y=y;
	//			pt.z=z;
	//			cell=cellFieldG->get(pt);
	//			currentConcentration=concentrationFieldPtr->get(pt);
	//			if(cell){ //do not secrete in medium				
	//				//mitr=secrData.typeIdSecrConstMap.find(cell->type);
	//						
	//					cerr<<"BEFORE SOLVING pt="<<pt<<" current concentration="<<currentConcentration<<endl;						
	//					cerr<<"sizeof(real)="<<sizeof(real)<<endl;
	//						
	//			}else{
	//				if(currentConcentration!=0.0){
	//					cerr<<"NON ZERO CONCENTRATION BEFORE SOLVING at pt="<<pt<<" current concentration="<<currentConcentration<<endl;						
	//				}
	//			}
	//		}

	//	int c=0;
 //		for (int i =0 ; i<containerRef.size() ; ++i){
	//		if(containerRef[i]!=0.0){
	//			cerr<<"containerRef["<<i<<"]="<<containerRef[i]<<endl;
	//			++c;				
	//		}
	//		if (c>6000)
	//			break;

	//	}



//call steady state diffusion equation solver

//#if defined(_WIN32)
//	volatile DWORD dwStart;
//	dwStart = GetTickCount();
//#endif



hw3crt_(&x_min, &x_max, &l, &lbdcnd, bda, bdb, 
		&y_min, &y_max, &m, &mbdcnd, bdc, bdd,
		&z_min, &z_max, &n, &nbdcnd, bde, bdf, 
		&elmbda, 
		&ldimf,&mdimf,
		f, 		 
		&pertrb, &ierror, w); 


//#if defined(_WIN32)
//	cerr<<"SOLVING PDE TOOK "<<GetTickCount()-dwStart<<" miliseconds to complete"<<endl;
//	dwStart = GetTickCount();
//#endif

if(ierror){
	cerr<<"solution has a problem. Error code: "<<ierror<<endl;
}

	//CellG *cell;
	//Point3D pt;
	//float currentConcentration;
	//for (int x = 0; x < fieldDim.x; x++)
	//	for (int y = 0; y < fieldDim.y; y++)
	//		for (int z = 0; z < fieldDim.z; z++){
	//			pt.x=x;
	//			pt.y=y;
	//			pt.z=z;
	//			cell=cellFieldG->get(pt);
	//			if(cell){ //do not secrete in medium				
	//				//mitr=secrData.typeIdSecrConstMap.find(cell->type);
	//				
	//				
	//					
	//					currentConcentration=concentrationFieldPtr->get(pt);
	//		
	//					cerr<<"AFTER SOLVING pt="<<pt<<" current concentration="<<currentConcentration<<endl;						
	//						
	//			}
	//		}



//ofstream outSolved("solvedHPP.txt");

//for (int j = 0 ; j < y_max ; ++j)
//	for (int i = 0 ; i < x_max ; ++i){
//		//idx=(i+1)*(m+1)+j+1;
//		//idx=(j+1)*(n+1)+i+1;
//		ix=(j+1)*(x_max+1)+i+1;
//		outSolved<<i<<" " <<j<<" 0 "<<f[ix]<<endl;
//	}

	//// cerr<<" DIFFUSE"<<endl;

	//// clock_t start, end;
	//// double time;
	//// start = clock();
	///// 'n' denotes neighbor

	/////this is the diffusion equation 
	/////C_{xx}+C_{yy}+C_{zz}=(1/a)*C_{t}
	/////a - diffusivity - diffConst

	/////Finite difference method:
	/////T_{0,\delta \tau}=F*\sum_{i=1}^N T_{i}+(1-N*F)*T_{0}
	/////N - number of neighbors
	/////will have to double check this formula


	//// Point3D pt, n;
	//// unsigned int token = 0;
	//// double distance;
	//CellG * currentCellPtr=0,*nCell=0;

	//// short currentCellType=0;
	//float concentrationSum=0.0;
	//float updatedConcentration=0.0;


	//float currentConcentration=0.0;
	//// short neighborCounter=0;

	//DiffusionData & diffData = diffSecrFieldTuppleVec[idx].diffData;
	//float diffConst=diffData.diffConst;
	//float decayConst=diffData.decayConst;
	//float deltaT=diffData.deltaT;
	//float deltaX=diffData.deltaX;
	//float dt_dx2=deltaT/(deltaX*deltaX);



	//// std::set<unsigned char>::iterator sitr;
	//// std::set<unsigned char>::iterator end_sitr=diffDataVec[idx].avoidTypeIdSet.end();
	//// std::set<unsigned char>::iterator end_sitr_decay=diffDataVec[idx].avoidDecayInIdSet.end();

	//// Automaton *automaton=potts->getAutomaton();

	//ConcentrationField_t * concentrationFieldPtr=concentrationFieldVector[idx];
	//ConcentrationField_t * scratchFieldPtr;

	////    scratchFieldPtr=concentrationFieldVector[diffDataVec.size()];
	//if(!haveCouplingTerms)
	//	scratchFieldPtr=concentrationFieldVector[diffSecrFieldTuppleVec.size()];
	//else
	//	scratchFieldPtr=concentrationFieldVector[diffSecrFieldTuppleVec.size()+idx];

	//Array2D_t & concentrationArray = concentrationFieldPtr->getContainer();
	//Array2D_t & scratchArray = scratchFieldPtr->getContainer();
	//boundaryConditionInit(concentrationFieldPtr);//initializing boundary conditions

	////    cerr<<" concentrationArray[1][1] = "<<concentrationArray[1][1]<<endl;
	////    cerr<<"concentrationFieldPtr->get(Point3D(0,0,0))="<<concentrationFieldPtr->get(Point3D(0,0,0))<<endl;

	//std::set<unsigned char>::iterator end_sitr_decay=diffData.avoidDecayInIdSet.end();   
	//bool avoidDecayInMedium=false;
	////the assumption is that medium has type ID 0

	//if(diffData.avoidDecayInIdSet.find(automaton->getTypeId("Medium")) != end_sitr_decay){
	//	avoidDecayInMedium=true;
	//}



	//unsigned int offset=workFieldDim.x*workFieldDim.y;
	//unsigned int counter=offset;
	//float *temp;

	//unsigned x_min=1,x_max=workFieldDim.x-1;
	//unsigned y_min=1,y_max=workFieldDim.y-1;

	//if(diffData.useBoxWatcher){
	//	Point3D minCoordinates=*(boxWatcherSteppable->getMinCoordinatesPtr());
	//	Point3D maxCoordinates=*(boxWatcherSteppable->getMaxCoordinatesPtr());

	//	x_min=minCoordinates.x+1;
	//	x_max=maxCoordinates.x+1;
	//	y_min=minCoordinates.y+1;
	//	y_max=maxCoordinates.y+1;
	//}

	////       for (int x = 1; x < workFieldDim.x-1; ++x)
	////          for (int y = 1; y < workFieldDim.y-1; ++y){
	//for (int x = x_min; x < x_max; ++x)
	//	for (int y = y_min; y < y_max; ++y){

	//		Point3D pt(x-1,y-1,0);
	//		currentCellPtr=cellFieldG->get(pt);
	//		currentConcentration = concentrationArray[x][y];
	//		temp=&scratchArray[x][y];
	//		//          temp  = &scratchVec[x][y][1];
	//		//          temp=&scratch[counter];
	//		//          scratchArray[x][y][1]=0;

	//		updatedConcentration=0.0;
	//		concentrationSum=0.0;

	//		concentrationSum+=concentrationArray[x][y+1];
	//		concentrationSum+=concentrationArray[x+1][y];
	//		concentrationSum+=concentrationArray[x][y-1];
	//		concentrationSum+=concentrationArray[x-1][y];

	//		updatedConcentration =  dt_dx2*diffConst*(concentrationSum - 4*currentConcentration)+currentConcentration;



	//		//processing decay depandent on type of the current cell

	//		//          updatedConcentration-=deltaT*(decayConst*currentConcentration);//decay in this type is allowed
	//		if(currentCellPtr){
	//			if(diffData.avoidDecayInIdSet.find(currentCellPtr->type)!=end_sitr_decay){
	//				;//decay in this type is forbidden
	//			}else{
	//				updatedConcentration-=deltaT*(decayConst*currentConcentration);//decay in this type is allowed
	//			}
	//		}else{
	//			if(avoidDecayInMedium){
	//				;//decay in Medium is forbidden
	//			}else{
	//				updatedConcentration-=deltaT*(decayConst*currentConcentration); //decay in Medium is allowed
	//			}
	//		}
	//		//          if(!currentCellPtr){
	//		//             updatedConcentration-=deltaT*(decayConst*currentConcentration);//decay in this type is allowed  
	//		//          }
	//		//          if (x<=3&&y<=3){
	//		//             cerr<<"concentrationSum="<<concentrationSum<<" updatedConcentration="<<updatedConcentration<<endl;
	//		//          }

	//		*temp=updatedConcentration;
	//		// 	         scratch[counter]=updatedConcentration;
	//		//             ++counter;
	//		//          scratchArray[x][y][1]=updatedConcentration;//updating scratch

	//	}


}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SteadyStateDiffusionSolver::diffuse() {

	for(unsigned int i = 0 ; i < diffSecrFieldTuppleVec.size() ; ++i ){
		diffuseSingleField(i);
		//if(!haveCouplingTerms){ //without coupling terms field swap takes place immediately aftera given field has been diffused
		//	ConcentrationField_t * concentrationField=concentrationFieldVector[i];
		//	ConcentrationField_t * scratchField=concentrationFieldVector[diffSecrFieldTuppleVec.size()];
		//	//copy updated values from scratch to concentration field
		//	// scrarch2Concentration(scratchField,concentrationField);
		//}

	}


}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// void SteadyStateDiffusionSolver::scrarch2Concentration( ConcentrationField_t *scratchField, ConcentrationField_t *concentrationField){
	// scratchField->switchContainersQuick(*(concentrationField));

// }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SteadyStateDiffusionSolver::outputField( std::ostream & _out, ConcentrationField_t *_concentrationField){
	Point3D pt;
	float tempValue;


	for (pt.x = 0; pt.x < fieldDim.x; pt.x++)
		for (pt.y = 0; pt.y < fieldDim.y; pt.y++)
			for (pt.z = 0; pt.z < fieldDim.z; pt.z++){
				tempValue=_concentrationField->get(pt);
				_out<<pt.x<<" "<<pt.y<<" "<<pt.z<<" "<<tempValue<<endl;
			}
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SteadyStateDiffusionSolver::readConcentrationField(std::string fileName,ConcentrationField_t *concentrationField){

	ifstream in(fileName.c_str());

	ASSERT_OR_THROW(string("Could not open chemical concentration file '") +
		fileName + "'!", in.is_open());

	Point3D pt;
	float c;
	//Zero entire field
	for (pt.z = 0; pt.z < fieldDim.z; pt.z++)
		for (pt.y = 0; pt.y < fieldDim.y; pt.y++)
			for (pt.x = 0; pt.x < fieldDim.x; pt.x++){
				concentrationField->set(pt,0);
			}  

			while(!in.eof()){
				in>>pt.x>>pt.y>>pt.z>>c;
				if(!in.fail())
					concentrationField->set(pt,c);
			}

}


void SteadyStateDiffusionSolver::update(CC3DXMLElement *_xmlData, bool _fullInitFlag){


	if(potts->getDisplayUnitsFlag()){
		Unit diffConstUnit=powerUnit(potts->getLengthUnit(),2)/potts->getTimeUnit();
		Unit decayConstUnit=1/potts->getTimeUnit();
	    Unit secretionConstUnit=1/potts->getTimeUnit();

		CC3DXMLElement * unitsElem=_xmlData->getFirstElement("Units"); 
		if (!unitsElem){ //add Units element
			unitsElem=_xmlData->attachElement("Units");
		}

		if(unitsElem->getFirstElement("DiffusionConstantUnit")){
			unitsElem->getFirstElement("DiffusionConstantUnit")->updateElementValue(diffConstUnit.toString());
		}else{
			unitsElem->attachElement("DiffusionConstantUnit",diffConstUnit.toString());
		}

		if(unitsElem->getFirstElement("DecayConstantUnit")){
			unitsElem->getFirstElement("DecayConstantUnit")->updateElementValue(decayConstUnit.toString());
		}else{
			unitsElem->attachElement("DecayConstantUnit",decayConstUnit.toString());
		}

		if(unitsElem->getFirstElement("DeltaXUnit")){
			unitsElem->getFirstElement("DeltaXUnit")->updateElementValue(potts->getLengthUnit().toString());
		}else{
			unitsElem->attachElement("DeltaXUnit",potts->getLengthUnit().toString());
		}

		if(unitsElem->getFirstElement("DeltaTUnit")){
			unitsElem->getFirstElement("DeltaTUnit")->updateElementValue(potts->getTimeUnit().toString());
		}else{
			unitsElem->attachElement("DeltaTUnit",potts->getTimeUnit().toString());
		}

		// if(unitsElem->getFirstElement("CouplingCoefficientUnit")){
			// unitsElem->getFirstElement("CouplingCoefficientUnit")->updateElementValue(decayConstUnit.toString());
		// }else{
			// unitsElem->attachElement("CouplingCoefficientUnit",decayConstUnit.toString());
		// }



		if(unitsElem->getFirstElement("SecretionUnit")){
			unitsElem->getFirstElement("SecretionUnit")->updateElementValue(secretionConstUnit.toString());
		}else{
			unitsElem->attachElement("SecretionUnit",secretionConstUnit.toString());
		}

		// if(unitsElem->getFirstElement("SecretionOnContactUnit")){
			// unitsElem->getFirstElement("SecretionOnContactUnit")->updateElementValue(secretionConstUnit.toString());
		// }else{
			// unitsElem->attachElement("SecretionOnContactUnit",secretionConstUnit.toString());
		// }

		// if(unitsElem->getFirstElement("ConstantConcentrationUnit")){
			// unitsElem->getFirstElement("ConstantConcentrationUnit")->updateElementValue(secretionConstUnit.toString());
		// }else{
			// unitsElem->attachElement("ConstantConcentrationUnit",secretionConstUnit.toString());
		// }

		if(unitsElem->getFirstElement("DecayConstantUnit")){
			unitsElem->getFirstElement("DecayConstantUnit")->updateElementValue(decayConstUnit.toString());
		}else{
			unitsElem->attachElement("DecayConstantUnit",decayConstUnit.toString());
		}

		if(unitsElem->getFirstElement("DeltaXUnit")){
			unitsElem->getFirstElement("DeltaXUnit")->updateElementValue(potts->getLengthUnit().toString());
		}else{
			unitsElem->attachElement("DeltaXUnit",potts->getLengthUnit().toString());
		}

		if(unitsElem->getFirstElement("DeltaTUnit")){
			unitsElem->getFirstElement("DeltaTUnit")->updateElementValue(potts->getTimeUnit().toString());
		}else{
			unitsElem->attachElement("DeltaTUnit",potts->getTimeUnit().toString());
		}

		// if(unitsElem->getFirstElement("CouplingCoefficientUnit")){
			// unitsElem->getFirstElement("CouplingCoefficientUnit")->updateElementValue(decayConstUnit.toString());
		// }else{
			// unitsElem->attachElement("CouplingCoefficientUnit",decayConstUnit.toString());
		// }

		if(unitsElem->getFirstElement("UptakeUnit")){
			unitsElem->getFirstElement("UptakeUnit")->updateElementValue(decayConstUnit.toString());
		}else{
			unitsElem->attachElement("UptakeUnit",decayConstUnit.toString());
		}

		if(unitsElem->getFirstElement("RelativeUptakeUnit")){
			unitsElem->getFirstElement("RelativeUptakeUnit")->updateElementValue(decayConstUnit.toString());
		}else{
			unitsElem->attachElement("RelativeUptakeUnit",decayConstUnit.toString());
		}

		if(unitsElem->getFirstElement("MaxUptakeUnit")){
			unitsElem->getFirstElement("MaxUptakeUnit")->updateElementValue(decayConstUnit.toString());
		}else{
			unitsElem->attachElement("MaxUptakeUnit",decayConstUnit.toString());
		}



	}


	//notice, only basic steering is enabled for PDE solvers - changing diffusion constants, do -not-diffuse to types etc...
	// Coupling coefficients cannot be changed and also there is no way to allocate extra fields while simulation is running
	diffSecrFieldTuppleVec.clear();

	CC3DXMLElementList diffFieldXMLVec=_xmlData->getElements("DiffusionField");
	for(unsigned int i = 0 ; i < diffFieldXMLVec.size() ; ++i ){
		diffSecrFieldTuppleVec.push_back(DiffusionSecretionFastFieldTupple3D());
		DiffusionData & diffData=diffSecrFieldTuppleVec[diffSecrFieldTuppleVec.size()-1].diffData;
		SecretionData & secrData=diffSecrFieldTuppleVec[diffSecrFieldTuppleVec.size()-1].secrData;
		vector<OxygenSecretionParameters> & oxygenSecretionData= diffSecrFieldTuppleVec[diffSecrFieldTuppleVec.size()-1].oxygenSecretionData;

		if(diffFieldXMLVec[i]->findElement("DiffusionData"))
			diffData.update(diffFieldXMLVec[i]->getFirstElement("DiffusionData"));

		if(diffFieldXMLVec[i]->findElement("SecretionData"))
			secrData.update(diffFieldXMLVec[i]->getFirstElement("SecretionData"));

		if(diffFieldXMLVec[i]->findElement("OxygenSecretion")){
			CC3DXMLElementList oxSecretionVec=diffFieldXMLVec[i]->getFirstElement("OxygenSecretion")->getElements("Parameters");
			std::map<char,OxygenSecretionParameters> oxSecretionParamsMap;

			for(unsigned  j = 0; j < oxSecretionVec.size() ; ++j){
				
				diffSecrFieldTuppleVec[diffSecrFieldTuppleVec.size()-1].useOxygenSecretion=true; //enable oxygen secretion

				OxygenSecretionParameters params;

				unsigned char cellType;				

				if(oxSecretionVec[j]->findAttribute("CellType")){
					cellType=automaton->getTypeId(oxSecretionVec[j]->getAttribute("CellType"));
					params.dataInitialized=true; //flag used to quickly determine if oxygen secretion data has been initialized properly
				}

				if(oxSecretionVec[j]->findAttribute("bf")){
					params.bf=oxSecretionVec[j]->getAttributeAsDouble("bf");
				}

				if(oxSecretionVec[j]->findAttribute("pblood")){
					params.pblood=oxSecretionVec[j]->getAttributeAsDouble("pblood");
				}

				if(oxSecretionVec[j]->findAttribute("beta")){
					params.beta=oxSecretionVec[j]->getAttributeAsDouble("beta");
				}

				if(oxSecretionVec[j]->findAttribute("n")){
					params.n=oxSecretionVec[j]->getAttributeAsDouble("n");
				}

				if(oxSecretionVec[j]->findAttribute("Khem")){
					params.Khem=oxSecretionVec[j]->getAttributeAsDouble("Khem");
				}

				if(oxSecretionVec[j]->findAttribute("alpha")){
					params.alpha=oxSecretionVec[j]->getAttributeAsDouble("alpha");
				}

				if(oxSecretionVec[j]->findAttribute("Hb")){
					params.Hb=oxSecretionVec[j]->getAttributeAsDouble("Hb");
				}

				if(oxSecretionVec[j]->findAttribute("delta")){
					params.delta=oxSecretionVec[j]->getAttributeAsDouble("delta");
				}

				
				oxSecretionParamsMap.insert(make_pair(cellType,params));

			}
			
			int oxygenSecretionDataSize=(--(oxSecretionParamsMap.end()))->first + 1 ; //map is ordered by cell type id value (highest cell type id last)


			oxygenSecretionData.assign(oxygenSecretionDataSize,OxygenSecretionParameters());

			for(map<char,OxygenSecretionParameters>::iterator mitr=oxSecretionParamsMap.begin() ; mitr!=oxSecretionParamsMap.end() ; ++mitr){
				oxygenSecretionData[mitr->first]=mitr->second;
			}

		}


		if(diffFieldXMLVec[i]->findElement("ReadFromFile"))
			readFromFileFlag=true;


	}
	if(_xmlData->findElement("Serialize")){

		serializeFlag=true;
		if(_xmlData->getFirstElement("Serialize")->findAttribute("Frequency")){
			serializeFrequency=_xmlData->getFirstElement("Serialize")->getAttributeAsUInt("Frequency");
		}
		cerr<<"serialize Flag="<<serializeFlag<<endl;

	}

	if(_xmlData->findElement("ReadFromFile")){
		readFromFileFlag=true;
		cerr<<"readFromFileFlag="<<readFromFileFlag<<endl;
	}





	for(int i = 0 ; i < diffSecrFieldTuppleVec.size() ; ++i){
		diffSecrFieldTuppleVec[i].diffData.setAutomaton(automaton);
		diffSecrFieldTuppleVec[i].secrData.setAutomaton(automaton);
		diffSecrFieldTuppleVec[i].diffData.initialize(automaton);
		diffSecrFieldTuppleVec[i].secrData.initialize(automaton);
	}

	///assigning member method ptrs to the vector

	for(unsigned int i = 0 ; i < diffSecrFieldTuppleVec.size() ; ++i){

		diffSecrFieldTuppleVec[i].secrData.secretionFcnPtrVec.assign(diffSecrFieldTuppleVec[i].secrData.secrTypesNameSet.size(),0);
		unsigned int j=0;
		for(set<string>::iterator sitr=diffSecrFieldTuppleVec[i].secrData.secrTypesNameSet.begin() ; sitr != diffSecrFieldTuppleVec[i].secrData.secrTypesNameSet.end()  ; ++sitr){

			if((*sitr)=="Secretion"){
				diffSecrFieldTuppleVec[i].secrData.secretionFcnPtrVec[j]=&SteadyStateDiffusionSolver::secreteSingleField;
				++j;
			}
			// else if((*sitr)=="SecretionOnContact"){
				// diffSecrFieldTuppleVec[i].secrData.secretionFcnPtrVec[j]=&SteadyStateDiffusionSolver::secreteOnContactSingleField;
				// ++j;
			// }
			// else if((*sitr)=="ConstantConcentration"){
				// diffSecrFieldTuppleVec[i].secrData.secretionFcnPtrVec[j]=&SteadyStateDiffusionSolver::secreteConstantConcentrationSingleField;
				// ++j;
			// }

		}
	}
}

std::string SteadyStateDiffusionSolver::toString(){
	return "SteadyStateDiffusionSolver";
}



std::string SteadyStateDiffusionSolver::steerableName(){
	return toString();
}

